import java.io.*;

/** 
 *  UsaProxy - HTTP proxy for tracking and logging of user interactions on websites
    Copyright (C) 2006  Monika Wnuk - Media Informatics Group at the University of Munich

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

 * 
 *  Class HTTPTraffic is used for 
 *  exclusively manage the current HTTP traffic data index that is used as server response identifier
 *  and for store requests and responses to individual txt-files (according to index)
 */
public class HTTPTraffic {
	
	private int 		index;
	private boolean 	logHttpTraffic;		/**  defines if HTTP traffic is logged or not
    										  *  stored in logHttpTraffic.txt */
	
	public HTTPTraffic() {
		doHttpTrafficLogging();
		if(logHttpTraffic) this.index = getHttpTrafficIndex(false);
	}
	
	/** exclusively get HTTP traffic index */
	public synchronized int getHttpTrafficIndex(boolean fromClient) {
		
		byte buffer[] = new byte[80];
		
		int clientIndex = 0;
    	
    	try {
    		
    		/** Open a stream to the HTTP traffic file. */
			FileInputStream fis = new FileInputStream ( "httpTraffic/httpTraffic.txt");

       		int len = fis.read( buffer, 0, fis.available() );
       		String str = new String( buffer, 0, len );
       		
       		clientIndex = Integer.parseInt(str);
       		
			fis.close();

    	}
    	catch ( FileNotFoundException e ) { 
        	/** If httpTraffic file doesn't exist, display error message. */
        	System.err.println("\nAn ERROR occured: httpTraffic file not found:\n"
									+ e );
									
			try{
				/** Create httpTraffic file. */
				FileOutputStream fos = new FileOutputStream ( "httpTraffic/httpTraffic.txt" , false);
				
				fos.write(("0").getBytes());
				clientIndex = 0;
				fos.flush();
				fos.close();
				System.err.println("httpTraffic file was created with index 0");
			}
			catch ( IOException ie ) { 	
        		System.err.println("\nAn ERROR occured while creating httpTraffic:\n"
							+ ie );
        	}	
			
        }
        
        catch ( IOException ie ) { 	
        	System.err.println("\nAn ERROR occured while getting HTTPTraffic index:\n"
							+ ie );
        }
        
        index = clientIndex;
        if(fromClient) increaseHttpTrafficIndex();
        
        /** notify waiting clients that index file is accessible */
		notifyAll();
        return clientIndex;
    	
    }
    
    /** get boolean from logHttpTraffic file if http traffic logging has to be performed or not */
	protected void doHttpTrafficLogging() {
		
		byte buffer[] = new byte[80];
		
		index = 0;
    	
    	try {
    		
    		/** Open a stream to the logHttpTraffic file. */
			FileInputStream fis = new FileInputStream ( "httpTraffic/logHttpTraffic.txt");

       		int len = fis.read( buffer, 0, fis.available() );
       		String str = new String( buffer, 0, len );
       		
       		Boolean doLog 	= new Boolean(str);
       		logHttpTraffic 	= doLog.booleanValue();
       		
			fis.close();

    	}
    	catch ( FileNotFoundException e ) { 
        	/** If httpTraffic file doesn't exist, display error message. */
        	System.err.println("\nAn ERROR occured: httpTraffic file not found:\n"
									+ e );
									
			try{
				/** Create httpTraffic file. */
				FileOutputStream fos = new FileOutputStream ( "httpTraffic/logHttpTraffic.txt" , false);
				
				fos.write(("true").getBytes());
	       		this.logHttpTraffic = true;
				fos.flush();
				fos.close();
				System.err.println("logHttpTraffic file was created with value true");
			}
			catch ( IOException ie ) { 	
        		System.err.println("\nAn ERROR occured while creating httpTraffic:\n"
							+ ie );
        	}	
			
        }
        
        catch ( IOException ie ) { 	
        	System.err.println("\nAn ERROR occured while getting HTTPTraffic index:\n"
							+ ie );
        }
    	
    }
    
    /** increase serverdata index */
	protected void increaseHttpTrafficIndex() {
    	
    	try {
    		
    		/** Open a stream to the HTTP traffic file. */
			FileOutputStream fos = new FileOutputStream ( "httpTraffic/httpTraffic.txt" , false);

       		fos.write((""+(this.index + 1)).getBytes());
       		this.index += 1;
			fos.flush();
			fos.close();

    	}
    	catch ( FileNotFoundException e ) { 
        	/** If httpTraffic file doesn't exist, display error message. */
        	System.err.println("\nAn ERROR occured: httpTraffic file not found:\n"
									+ e );
        }
        
        catch ( IOException ie ) { 	
        	System.err.println("\nAn ERROR occured while increasing httpTraffic index:\n"
							+ ie );
        }
        /** notify waiting clients that index file is accessible */
		notifyAll();
    	
    }
    
    /** store serverdata to an individual file */
	public void store (int index, String data) {
    	
    	try {
    		
    		/** Open a stream to the individual file. */
			FileOutputStream fos = new FileOutputStream ( "httpTraffic/log/httpTraffic"+index+".txt" , true);
			
			fos.write(data.getBytes(HTTPConstants.HEADER_DEFAULT_CHAR_ENCODING));
			fos.flush();
			fos.close();

    	}
    	catch ( FileNotFoundException e ) { 
        	System.err.println("\nAn ERROR occured: txt file with index " + index + " not found:\n"
									+ e );
        }
        
        catch ( IOException ie ) { 	
        	System.err.println("\nAn ERROR occured while storing data to file with index " + index + ":\n"
							+ ie );
        }
    	
    }
    
    /** store serverdata to an individual file */
	public void store (int index, byte[] data, int off, int length) {
    	
    	try {
    		
    		/** Open a stream to the individual file. */
			FileOutputStream fos = new FileOutputStream ( "httpTraffic/log/httpTraffic"+index+".txt" , true);
			
			fos.write(data, off, length);
			fos.flush();
			fos.close();

    	}
    	catch ( FileNotFoundException e ) { 
        	System.err.println("\nAn ERROR occured: txt file with index " + index + " not found:\n"
									+ e );
        }
        
        catch ( IOException ie ) { 	
        	System.err.println("\nAn ERROR occured while storing data to file with index " + index + ":\n"
							+ ie );
        }
    	
    }
    
    

	/** Returns the logHttpTraffic.*/
	public boolean isLogHttpTraffic() {
		return logHttpTraffic;
	}

}
