import java.io.*;
import java.net.Socket;

/** 
 *  UsaProxy - HTTP proxy for tracking and logging of user interactions on websites
    Copyright (C) 2006  Monika Wnuk - Media Informatics Group at the University of Munich

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

 *  Class LogFile is used for synchronizing the log file access
 *  Method log() can only be called by one ClientRequest at the same time
 */
public class LogFile {
	
	/** exclusively writing log records to the log file */
	public synchronized void log (OutputStream out, String data, Socket client) {
    	
    	/** if client log request (else: serverdata log-request from proxy)*/
    	if(out!=null)
    		out = new DataOutputStream(out);
    	
    	try {
    		
    		/** Open a stream to the log-file. */
			FileOutputStream fos = new FileOutputStream ( "log.txt" , true);
			
			String clientIP = client.getInetAddress().getHostAddress();
			
			String[] dataArray;
			
			dataArray = data.split("--");
			data = "";
			for (int m=0; m<dataArray.length; m++) {
				data = data + clientIP + " " + dataArray[m] + "\r\n";
			}
			data = data.replaceAll("%20", " ");
			
			fos.write(data.getBytes());
			fos.flush();
			fos.close();
			
			if(out!=null) {
				/** Since client expects a response to his request
				 *  send 404 error message to client */
				PrintWriter outPrint = new PrintWriter(new OutputStreamWriter(out));
				outPrint.println("HTTP/1.0 404 ");            /** version and status code */
				outPrint.println();                           /** blank line */
				outPrint.flush();
			}

    	}
    	catch ( FileNotFoundException e ) { 
        	/** If log file doesn't exist, display the famous 404 message. */
        	System.err.println("\nAn ERROR occured: log file not found:\n"
									+ e );
        	
        	/** Send 404 error message to client */
			PrintWriter outPrint = new PrintWriter(new OutputStreamWriter(out));
			outPrint.println("HTTP/1.0 404 ");            /** version and status code */
			outPrint.println();                           /** blank line */
			outPrint.flush();
        }
        
        catch ( IOException ie ) { 	
        	System.err.println("\nAn ERROR occured while logging:\n"
							+ ie );
        }
        
        /** notify waiting clients that log file is accessible */
		notifyAll();
    	
    }

}
