import java.io.*;
import java.net.Socket;

/** 
 *  UsaProxy - HTTP proxy for tracking and logging of user interactions on websites
    Copyright (C) 2006  Monika Wnuk - Media Informatics Group at the University of Munich

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

 *  Class ProxyScript is used for the synchronized forwarding of proxyscript.js
 *  Method send() can only be called by one ClientRequest at the same time
 */
public class ProxyScript {
	
	/** exclusively sending the proxyscript-file to the client 
     */
	public synchronized void send (OutputStream out, String file, Socket client) {
    	
    	out = new DataOutputStream(out);
		try { 

			/** Open a stream to the file. Remember that by this
			 *  all the text but the file name has been stripped 
			 *  from the "request" string. */
			FileInputStream fis = new FileInputStream ( file ); 
			
			/** Creat a byte array to hold the file. */
			byte [] data = new byte [ fis.available() ]; 
			
			fis.read( data ); /** Read file into the byte array */
			out.write( data );/** Write it to client output stream */
			out.flush();      /** Remember to flush output buffer */
        }
        catch ( FileNotFoundException e ) { 	
        	/** If no such file, then display the famous 404 message. */
        	/** If log file doesn't exist, display the famous 404 message. */
        	System.err.println("\nAn ERROR occured: script file not found:\n"
									+ e );
        	
        	/** Send 404 error message to client */
			PrintWriter outPrint = new PrintWriter(new OutputStreamWriter(out));
			outPrint.println("HTTP/1.0 404 ");            /** version and status code */
			outPrint.println();                           /** blank line */
			outPrint.flush();
	
        }
        catch ( IOException e ) { 
        	System.err.println("\nAn ERROR occured while sending script file:\n"
							+ e );
        }
        
        /** notify waiting clients that script file is accessible */
        notifyAll();
    	
    }

}
