/** 
 *  UsaProxy - HTTP proxy for tracking and logging of user interactions on websites
    Copyright (C) 2006  Monika Wnuk - Media Informatics Group at the University of Munich

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

 * Class ServerRequest forwards client request to server
 * Thread due to asynchronity of requests and responses
 */

import java.io.*; 
import java.util.*;

public class ServerRequest extends Thread {

    private OutputStream 	serverOut;
    private InputStream 	clientIn;
    private Hashtable 		requestHeaders;
    private ClientRequest	clientRequest;
    
    /** Constructor: create ServerRequest with a given output stream and the client request*/
    public ServerRequest (OutputStream serverOut, Hashtable requestHeaders,
    							ClientRequest clientRequest) throws Exception {

		this.serverOut 		= serverOut;
		this.clientIn 			= null;
		this.requestHeaders 	= requestHeaders;
		this.clientRequest 	= clientRequest;
	
		/** start thread */
		start();

    } 
    
    /** Constructor: create ServerRequest with a given output stream, the client request including POST entity*/
    public ServerRequest (OutputStream serverOut, Hashtable requestHeaders, InputStream clientIn,
    							ClientRequest clientRequest) throws Exception {

		this.serverOut 		= serverOut;
		this.clientIn 			= clientIn;
		this.requestHeaders 	= requestHeaders;
		this.clientRequest 	= clientRequest;
	
		/** start thread */
		start();

    } 
    
    /** run()-method which is called by start() */
    public void run () { 

		try { 
	
			serverOut = new DataOutputStream(new BufferedOutputStream(serverOut));
			StringBuffer header = new StringBuffer();
			
			for (java.util.Iterator e = requestHeaders.entrySet().iterator(); e.hasNext();) {
		
		        java.util.Map.Entry me = (java.util.Map.Entry) e.next();
		        Object keyObj = me.getKey();
		        if (null == keyObj) continue;
		        String key = keyObj.toString().trim();
		        if (key.toLowerCase().startsWith("get") || key.toLowerCase().startsWith("post") || key.toLowerCase().startsWith("connect "))
		        	header.insert(0,key + "\r\n");
		        else {
		        	String[] headerField = (String[])me.getValue();
		        	header.append(headerField[0]).append(": ").append(headerField[1]).append("\r\n");
		        }
		    }
			header.append("\r\n");
			
	/*		System.out.println();
			System.out.print(header.toString());
			System.out.println();   */
			
			serverOut.write(header.toString().getBytes(HTTPConstants.HEADER_DEFAULT_CHAR_ENCODING));
			serverOut.flush();	

			/** create new httpTraffic index for the request that will be
			 *  used for suffixing the txt-file the request and response data will be stored to
			 *  Additionally, store the request */
			if(clientRequest.getHttpTraffic().isLogHttpTraffic()) {
				clientRequest.getHttpTraffic().store(clientRequest.getHttpTrafficIndex(),"##### Request #####\r\n\r\n"+header.toString());
			}
			
			/** if any additional data was sent with the request headers (POST) */
			if(clientIn!=null) {	
				byte request[] = new byte[4096]; /** data buffer */  
	        	int requestLength;   /** length of request as number of bytes */
	        
	        	while ((requestLength = clientIn.read(request)) != -1) {
					if (requestLength > 0) {
				/**		for(int gg=0; gg<request.length; gg++)
						System.out.print((char)request[gg]);   */
			            serverOut.write (request,0,requestLength);
			            serverOut.flush();
			        
			        	/** store the request to txt-file */
			        	if(clientRequest.getHttpTraffic().isLogHttpTraffic()) 
							clientRequest.getHttpTraffic().store(clientRequest.getHttpTrafficIndex(), request, 0, requestLength);
			        }
				}
			}
			
			serverOut.flush();
		    
		} catch (IOException e) {
        	System.err.println("\nAn ERROR occured while sending request: "
							+ e );
		}
    }



}
